#!/usr/bin/env python
# Copyright (c) 2018 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""pynamedotcom package metadata."""

from __future__ import print_function
from __future__ import unicode_literals

__version__ = "0.1.0"
__author__ = "Ben Maddison"
__author_email__ = "benm@workonline.co.za"
__licence__ = "MIT"
__copyright__ = "Copyright (c) 2018 Ben Maddison"
__url__ = "https://github.com/benmaddison/pynamedotcom"
__classifiers__ = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Internet',
]
__entry_points__ = {
    'console_scripts': [
        'namedotcom=pynamedotcom.cli:main',
    ]
}


if __name__ == "__main__":
    print(__version__)



# Copyright (c) 2018 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""pynamedotcom API module."""

from __future__ import print_function
from __future__ import unicode_literals

import logging
import requests
from requests.auth import HTTPBasicAuth

from pynamedotcom.domain import Domain
from pynamedotcom.search import SearchResult


class API(object):
    """API client library class."""

    def __init__(self, user=None, token=None,
                 host="api.name.com", version=4):
        """Construct API instance."""
        self.base_url = "https://{}/v{}".format(host, version)
        self.auth = HTTPBasicAuth(user, token)

    def __enter__(self):
        """Enter context manager."""
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        """Exit context manager."""
        pass

    def _get(self, endpoint=None, params=None):
        """Make a HTTP GET request."""
        url = "{}/{}".format(self.base_url, endpoint)
        resp = requests.get(url, params=params, auth=self.auth)
        logging.getLogger(__name__).debug(resp.json())
        resp.raise_for_status()
        return resp

    def _post(self, endpoint=None, data=None):
        """Make a HTTP POST request."""
        url = "{}/{}".format(self.base_url, endpoint)
        resp = requests.post(url, json=data, auth=self.auth)
        logging.getLogger(__name__).debug(resp.json())
        resp.raise_for_status()
        return resp

    def ping(self):
        """Check service reachability."""
        resp = self._get(endpoint="hello")
        return resp.json()

    def domain(self, name):
        """Get a domain."""
        resp = self._get(endpoint="domains/{}".format(name))
        return Domain(session=self, **resp.json())

    @property
    def domains(self):
        """Get list of domains as a generator."""
        resp = self._get(endpoint="domains")
        if "domains" in resp.json():
            for domain in resp.json()["domains"]:
                yield domain["domainName"]
        else:
            return

    def check_availability(self, name):
        """Check domain name availablility."""
        search_data = {
            'domainNames': [name]
        }
        resp = self._post(endpoint="domains:checkAvailability",
                          data=search_data)
        for result in resp.json()['results']:
            if result['domainName'] == name:
                return SearchResult(session=self, **result)
        return False



# Copyright (c) 2018 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""pynamedotcom exceptions module."""

from __future__ import print_function
from __future__ import unicode_literals


class BaseException(Exception):
    """Base pynamedotcom exception class."""
    pass


class DomainUnlockTimeError(BaseException):
    """
    Error indicating that the specified domain cannot be unlocked until the
    specified time.
    """
    pass


class NameserverUpdateError(BaseException):
    """
    Error indicating a failure setting the delegation NS resource records on a
    domain. Usually because of missing "glue" records.
    """
    pass



# Copyright (c) 2018 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""pynamedotcom decorators module."""

from __future__ import print_function
from __future__ import unicode_literals

import functools


def require_type(typ):
    """Decorate func to check type of 'value' argument before calling."""
    def decorator(func):
        @functools.wraps(func)
        def wrapper(self, value, *args, **kwargs):
            if not isinstance(value, typ):
                raise TypeError(
                    "method {} requires value argument of type {}, got {} ({})"
                    .format(func.__name__, typ, value, type(value)))
            return func(self, value, *args, **kwargs)
        return wrapper
    return decorator


def readonly(func):
    """Decorate a property setter to raise make it read-only."""
    @functools.wraps(func)
    def wrapper(self, value, *args, **kwargs):
        raise AttributeError("attribute {} is read-only".format(func.__name__))
    return wrapper



# Copyright (c) 2018 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""pynamedotcom contact module."""

from __future__ import print_function
from __future__ import unicode_literals


ROLES = ["admin", "tech", "registrant", "billing"]


class Contact(object):
    """Contact class."""

    def __init__(self, session, firstName=None, lastName=None,
                 companyName=None, address1=None, address2=None,
                 city=None, state=None, zip=None, country=None,
                 phone=None, fax=None, email=None):
        """Construct SearchResult object instance."""
        self.session = session
        self._first_name = firstName
        self._last_name = lastName
        self._company_name = companyName
        self._address = {
            "street": [address1, address2],
            "city": city,
            "state": state,
            "zip": zip,
            "country": country
        }
        self._phone = phone
        self._fax = fax
        self._email = email

    def __repr__(self):
        if self.company_name:
            return "{} {} ({}) <{}>".format(self.first_name, self.last_name,
                                            self.company_name, self.email)
        else:
            return "{} {} <{}>".format(self.first_name, self.last_name,
                                       self.email)

    def __getattr__(self, name):
        """Get private attributes."""
        try:
            return self.__getattribute__("_{}".format(name))
        except AttributeError:
            raise AttributeError("{} object has no attribute {}"
                                 .format(self.__class__, name))



#!/usr/bin/env python
# Copyright (c) 2018 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""Setup configuration script for package."""

from __future__ import print_function
from __future__ import unicode_literals

import os
import re

from setuptools import find_packages, setup

package = {"__name__": find_packages()[0]}
here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, package["__name__"], "__meta__.py")) as f:
    exec(f.read(), package)

with open(os.path.join(here, "packaging", "requirements.txt")) as f:
    package["__requirements__"] = f.readlines()

with open(os.path.join(here, "README.md")) as f:
    for line in f:
        match = re.match(r'<!--description: () -->', line)
        if match is not None:
            package["__description__"] = match.group(1)
            break
    else:
        package["__description__"] = None

try:
    import pypandoc
    package["__readme__"] = pypandoc.convert_file('README.md',
                                                  'rst', format='md')
except (ImportError, RuntimeError) as e:
    print("README conversion failed: {}".format(e))
    package["__readme__"] = None


setup(
    name=package["__name__"],
    version=package["__version__"],
    author=package["__author__"],
    author_email=package["__author_email__"],
    description=package["__description__"],
    long_description=package["__readme__"],
    license=package["__licence__"],
    classifiers=package["__classifiers__"],
    packages=find_packages(),
    include_package_data=True,
    url=package["__url__"],
    download_url="{}/{}".format(package["__url__"], package["__version__"]),
    install_requires=package["__requirements__"],
    entry_points=package["__entry_points__"]
)



# Copyright (c) 2018 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""pynamedotcom Domain module."""

from __future__ import print_function
from __future__ import unicode_literals

import logging

from requests.exceptions import HTTPError

from pynamedotcom.contact import Contact
from pynamedotcom.decorators import readonly, require_type
from pynamedotcom.exceptions import (DomainUnlockTimeError,
                                     NameserverUpdateError)


class Domain(object):
    """Domain class."""

    def __init__(self, session, **kwargs):
        """Construct Domain object instance."""
        self.session = session
        self._set(**kwargs)

    def __repr__(self):
        return "{}({})".format(self.__class__.__name__, self.name)

    def _set(self, domainName, nameservers=None, contacts=None,
             privacyEnabled=False, locked=False, autorenewEnabled=False,
             expireDate=None, createDate=None, renewalPrice=0):
        """Set local properties."""
        self._name = domainName
        self._nameservers = nameservers
        self._privacy = privacyEnabled
        self._locked = locked
        self._autorenew = autorenewEnabled
        self._expiry = expireDate
        self._created = createDate
        self._renewal_price = renewalPrice
        self._contacts = {}
        for role, contact in contacts.items():
            self._contacts[role] = Contact(session=self.session, **contact)

    def refresh(self):
        """Retrieve domain properties."""
        resp = self.session._get(endpoint="domains/{}".format(self.name))
        self._set(**resp.json())
        return self

    @property
    def name(self):
        return self._name

    @name.setter
    @readonly
    def name(self, value):
        pass  # pragma: no cover

    @property
    def nameservers(self):
        return self._nameservers

    @nameservers.setter
    @require_type(list)
    def nameservers(self, value):
        logging.getLogger(__name__).debug("setting {}.nameservers = {}"
                                          .format(self, value))
        endpoint = "domains/{}:setNameservers".format(self.name)
        data = {
            "nameservers": value
        }
        try:
            resp = self.session._post(endpoint=endpoint, data=data)
            self._set(**resp.json())
        except HTTPError as e:
            resp = e.response
            data = resp.json()
            if resp.status_code == 500 \
                    and "Data Management Policy Violation" in data["details"]:
                raise NameserverUpdateError(data["details"])
            else:
                raise e

    @property
    def contacts(self):
        return self._contacts

    @contacts.setter
    def contacts(self, value):
        raise NotImplementedError

    @property
    def privacy(self):
        return self._privacy

    @privacy.setter
    def privacy(self, value):
        raise NotImplementedError

    @property
    def locked(self):
        return self._locked

    @locked.setter
    @require_type(bool)
    def locked(self, value):
        logging.getLogger(__name__).debug("setting {}.locked = {}"
                                          .format(self, value))
        if value:
            endpoint = "domains/{}:lock".format(self.name)
        else:
            endpoint = "domains/{}:unlock".format(self.name)
        try:
            resp = self.session._post(endpoint=endpoint)
            self._set(**resp.json())
        except HTTPError as e:
            resp = e.response
            data = resp.json()
            if resp.status_code == 400 \
                    and "Domain can not be unlocked until" in data["details"]:
                raise DomainUnlockTimeError(data["details"])
            else:
                raise e

    @property
    def autorenew(self):
        return self._autorenew

    @autorenew.setter
    @require_type(bool)
    def autorenew(self, value):
        logging.getLogger(__name__).debug("setting {}.autorenew = {}"
                                          .format(self, value))
        if value:
            endpoint = "domains/{}:enableAutorenew".format(self.name)
        else:
            endpoint = "domains/{}:disableAutorenew".format(self.name)
        resp = self.session._post(endpoint=endpoint)
        self._set(**resp.json())

    @property
    def expiry(self):
        return self._expiry

    @expiry.setter
    @readonly
    def expiry(self, value):
        pass  # pragma: no cover

    @property
    def created(self):
        return self._created

    @created.setter
    @readonly
    def created(self, value):
        pass  # pragma: no cover

    @property
    def renewal_price(self):
        return self._renewal_price

    @renewal_price.setter
    @readonly
    def renewal_price(self, value):
        pass  # pragma: no cover



# Copyright (c) 2018 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""pynamedotcom search module."""

from __future__ import print_function
from __future__ import unicode_literals


class SearchResult(object):
    """SearchResult class."""

    def __init__(self, session, domainName, sld, tld,
                 purchasable=False, premium=False, purchasePrice=None,
                 purchaseType=None, renewalPrice=None):
        """Construct SearchResult object instance."""
        self.session = session
        self._name = domainName
        self._sld = sld
        self._tld = tld
        self._purchasable = purchasable
        self._premium = premium
        self._purchase_price = purchasePrice
        self._purchase_type = purchaseType
        self._renewal_price = renewalPrice

    def __repr__(self):
        return "{}({})".format(self.__class__.__name__, self.name)

    def __getattr__(self, name):
        """Get private attributes."""
        try:
            return self.__getattribute__("_{}".format(name))
        except AttributeError:
            raise AttributeError("{} object has no attribute {}"
                                 .format(self.__class__, name))



# Copyright (c) 2018 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""pynamedotcom CLI module."""

from __future__ import print_function

import click
import json
import logging
import os

from argparse import Namespace

from pynamedotcom import API


logger = logging.getLogger(__name__)


def _default_auth_path():  # pragma: no cover
    """Try and find the default auth-file, if it exists."""
    if "XDG_CONFIG_HOME" in os.environ:
        base_path = os.environ.get("XDG_CONFIG_HOME")
    elif "HOME" in os.environ:
        base_path = os.path.join(os.environ.get("HOME"), ".config")
    else:
        return None
    path = os.path.join(base_path, "pynamedotcom", "auth.json")
    if os.path.exists(path):
        return path
    return None


def _set_log_level(ctx, param, value):
    """Set logging level according to the --debug flag."""
    if value:
        log_level = logging.DEBUG
    else:
        log_level = logging.WARNING
    logging.basicConfig(level=log_level)
    return value


@click.group()
@click.pass_context
@click.option("-d", "--debug", is_flag=True,
              callback=_set_log_level, is_eager=True, expose_value=False,
              help="Enable debug logging.")
@click.option("-h", "--host", default="api.name.com",
              help="Server hostname.", show_default=True)
@click.option("-u", "--username",
              help="name.com username. Overides --auth-file.")
@click.option('-t', "--token",
              help="name.com API token. Overides --auth-file")
@click.option("-f", "--auth-file", type=click.Path(exists=True),
              default=_default_auth_path(), show_default=True,
              help="Read credentials from file.")
@click.version_option()
def main(ctx, host, auth_file, username, token):
    """CLI tool for interacting with the name.com API."""
    # Get credentials from file or CLI options
    auth = {"user": None, "token": None}
    # Try reading from file
    if auth_file:
        logger.debug("reading auth parameters from {}".format(auth_file))
        with open(auth_file) as f:
            auth = json.load(f)
    # Overide values based on -u/-t options
    if username:
        auth["user"] = username
    if token:
        auth["token"] = token
    logger.debug("connecting to {} as {}".format(host, auth["user"]))

    # Declare helper function
    def api():
        """Helper function to return configured pynamedotcom.API instance."""
        return API(host=host, **auth)

    ctx.obj = Namespace()
    # Add helper to click Context.obj to pass to command functions
    ctx.obj.api = api


@main.command()
@click.pass_context
def ping(ctx):
    """Check for successful authentication."""
    # Use provided helper to instantiate pynamedotcom.API object
    with ctx.obj.api() as api:
        try:
            # Execute method and print a success message
            api.ping()
            click.echo(click.style("OK", fg="green"))
        except Exception as e:  # pragma: no cover
            # fail cleanly
            ctx.fail(message="{}".format(e))


@main.command()
@click.pass_context
def domains(ctx):
    """Get list of domain names."""
    # Use provided helper to instantiate pynamedotcom.API object
    with ctx.obj.api() as api:
        try:
            # Execute method and print a success message
            for domain in api.domains:
                click.echo(domain)
        except Exception as e:  # pragma: no cover
            # fail cleanly
            ctx.fail(message="{}".format(e))


@main.group(invoke_without_command=True)
@click.pass_context
@click.argument("name")
def domain(ctx, name):
    """Get domain details."""
    # Record args in Context
    ctx.obj.name = name
    # Only execute if no subcommand is provided
    if ctx.invoked_subcommand is None:
        # Use provided helper to instantiate pynamedotcom.API object
        with ctx.obj.api() as api:
            try:
                # Execute method and print the domain details
                domain = api.domain(name=name)
                click.echo("{}".format(domain.name))
                click.echo("  nameservers:")
                for ns in domain.nameservers:
                    click.echo("    {}".format(ns))
                click.echo("  contacts:")
                for role, contact in domain.contacts.items():
                    click.echo("    {}: {}".format(role, contact))
                click.echo("  privacy: {}".format(domain.privacy))
                click.echo("  locked: {}".format(domain.locked))
                click.echo("  autorenew: {}".format(domain.autorenew))
                click.echo("  expiry: {}".format(domain.expiry))
                click.echo("  created: {}".format(domain.created))
                click.echo("  renewal price: ${}".format(domain.renewal_price))
            except Exception as e:  # pragma: no cover
                # fail cleanly
                ctx.fail(message="{}".format(e))


@domain.command()
@click.pass_context
def name(ctx):
    """Get domain name."""
    # Use provided helper to instantiate pynamedotcom.API object
    with ctx.obj.api() as api:
        try:
            # Execute method and print the domain details
            domain = api.domain(name=ctx.obj.name)
            click.echo("{}".format(domain.name))
        except Exception as e:  # pragma: no cover
            # fail cleanly
            ctx.fail(message="{}".format(e))


@domain.command()
@click.pass_context
@click.argument("nameservers", nargs=-1, required=False)
def nameservers(ctx, nameservers):
    """Get or set domain nameservers."""
    # Coerce nameservers argument to list type
    nameservers = list(nameservers)
    # Use provided helper to instantiate pynamedotcom.API object
    with ctx.obj.api() as api:
        try:
            # Execute method and print the domain details
            domain = api.domain(name=ctx.obj.name)
            if nameservers:
                domain.nameservers = nameservers
                click.echo(click.style("OK", fg="green"))
            else:
                for nameserver in domain.nameservers:
                    click.echo("{}".format(nameserver))
        except Exception as e:  # pragma: no cover
            # fail cleanly
            ctx.fail(message="{}".format(e))


@domain.command()
@click.pass_context
def contacts(ctx):
    """Get domain contacts."""
    # Use provided helper to instantiate pynamedotcom.API object
    with ctx.obj.api() as api:
        try:
            # Execute method and print the domain details
            domain = api.domain(name=ctx.obj.name)
            for role, contact in domain.contacts.items():
                click.echo("{}: {}".format(role, contact))
        except Exception as e:  # pragma: no cover
            # fail cleanly
            ctx.fail(message="{}".format(e))


@domain.command()
@click.pass_context
def privacy(ctx):
    """Get domain privacy status."""
    # Use provided helper to instantiate pynamedotcom.API object
    with ctx.obj.api() as api:
        try:
            # Execute method and print the domain details
            domain = api.domain(name=ctx.obj.name)
            click.echo("{}".format(domain.privacy))
        except Exception as e:  # pragma: no cover
            # fail cleanly
            ctx.fail(message="{}".format(e))


@domain.command()
@click.pass_context
@click.argument("state", type=bool, required=False)
def locked(ctx, state):
    """Get or set domain lock status."""
    # Use provided helper to instantiate pynamedotcom.API object
    with ctx.obj.api() as api:
        try:
            # Execute method and print the domain details
            domain = api.domain(name=ctx.obj.name)
            if state is not None:
                domain.locked = state
                click.echo(click.style("OK", fg="green"))
            else:
                click.echo("{}".format(domain.locked))
        except Exception as e:  # pragma: no cover
            # fail cleanly
            ctx.fail(message="{}".format(e))


@domain.command()
@click.pass_context
@click.argument("state", type=bool, required=False)
def autorenew(ctx, state):
    """Get or set domain autorenew status."""
    # Use provided helper to instantiate pynamedotcom.API object
    with ctx.obj.api() as api:
        try:
            # Execute method and print the domain details
            domain = api.domain(name=ctx.obj.name)
            if state is not None:
                domain.autorenew = state
                click.echo(click.style("OK", fg="green"))
            else:
                click.echo("{}".format(domain.autorenew))
        except Exception as e:  # pragma: no cover
            # fail cleanly
            ctx.fail(message="{}".format(e))


@domain.command()
@click.pass_context
def expiry(ctx):
    """Get domain expiry date/time."""
    # Use provided helper to instantiate pynamedotcom.API object
    with ctx.obj.api() as api:
        try:
            # Execute method and print the domain details
            domain = api.domain(name=ctx.obj.name)
            click.echo("{}".format(domain.expiry))
        except Exception as e:  # pragma: no cover
            # fail cleanly
            ctx.fail(message="{}".format(e))


@domain.command()
@click.pass_context
def created(ctx):
    """Get domain creation date/time."""
    # Use provided helper to instantiate pynamedotcom.API object
    with ctx.obj.api() as api:
        try:
            # Execute method and print the domain details
            domain = api.domain(name=ctx.obj.name)
            click.echo("{}".format(domain.created))
        except Exception as e:  # pragma: no cover
            # fail cleanly
            ctx.fail(message="{}".format(e))


@domain.command()
@click.pass_context
def renewal_price(ctx):
    """Get domain renewal price."""
    # Use provided helper to instantiate pynamedotcom.API object
    with ctx.obj.api() as api:
        try:
            # Execute method and print the domain details
            domain = api.domain(name=ctx.obj.name)
            click.echo("${}".format(domain.renewal_price))
        except Exception as e:  # pragma: no cover
            # fail cleanly
            ctx.fail(message="{}".format(e))


@main.command()
@click.pass_context
@click.argument("name")
def search(ctx, name):
    """Search for domain availablility."""
    # Use provided helper to instantiate pynamedotcom.API object
    with ctx.obj.api() as api:
        try:
            # Execute method and print the results
            result = api.check_availability(name=name)
            if result.purchasable:
                click.echo(click.style("{} is available:".format(result.name),
                                       fg="green"))
                click.echo("  premium: {}".format(result.premium))
                click.echo("  type: {}".format(result.purchase_type))
                click.echo("  purchase price: ${}".format(result.purchase_price))  # noqa
                click.echo("  renewal price: ${}".format(result.renewal_price))
            else:
                click.echo(click.style("{} is not available".format(result.name),  # noqa
                                       fg="red"))
                ctx.exit(code=1)
        except Exception as e:  # pragma: no cover
            # fail cleanly
            ctx.fail(message="{}".format(e))
